import { Box, Button, Modal, TextField, Typography } from '@mui/material';
import { SubmitHandler, useForm } from 'react-hook-form';
import { CreateField } from '../../interfaces/create';

interface Props {
    title?: string;
    open: boolean;
    setOpen: (state: boolean) => void;
    submitHandler?: any;
    fields: CreateField[];
    submitButtonText?: string;
    mutateHandler?: any;
}

const style = {
    position: 'absolute',
    top: '50%',
    left: '50%',
    transform: 'translate(-50%, -50%)',
    width: 400,
    bgcolor: 'background.paper',
    boxShadow: 24,
    borderRadius: 2,
    p: 4,
    display: "flex",
    flexDirection: "column",
    gap: "8px"
}

function ModalCreate({
    title = '',
    open,
    setOpen,
    submitHandler,
    fields,
    submitButtonText = 'Сохранить',
    mutateHandler
}: Props) {
    const getDefaultValues = (fields: CreateField[]) => {
        let result: { [key: string]: string | boolean } = {}
        fields.forEach((field: CreateField) => {
            result[field.key] = field.defaultValue
        })
        return result
    }

    const { register, handleSubmit, formState: { errors } } = useForm({
        defaultValues: getDefaultValues(fields)
    })

    const onSubmit: SubmitHandler<any> = async (data) => {
        try {
            await submitHandler?.(data).then(() => {
                mutateHandler?.()
            })
        } catch (error) {
            console.error(error)
        }
    }

    return (
        <Modal
            open={open}
            onClose={() => setOpen(false)}
            aria-labelledby="modal-modal-title"
            aria-describedby="modal-modal-description"
        >
            <form onSubmit={handleSubmit(onSubmit)}>
                <Box sx={style}>
                    <Typography variant="h6" component="h6" gutterBottom>
                        {title}
                    </Typography>

                    {fields.map((field: CreateField) => {
                        return (
                            <TextField
                                key={field.key}
                                fullWidth
                                margin='normal'
                                label={field.headerName || field.key.charAt(0).toUpperCase() + field.key.slice(1)}

                                required={field.required || false}
                                {...register(field.key, { required: `${field.key} обязателен` })}
                                error={!!errors[field.key]}
                            />
                        )
                    })}

                    <Box sx={{
                        display: "flex",
                        justifyContent: "space-between",
                        gap: "8px"
                    }}>
                        <Button type="submit" variant="contained" color="primary">
                            {submitButtonText}
                        </Button>

                        <Button type="button" variant="outlined" color="primary" onClick={() => setOpen(false)}>
                            Отмена
                        </Button>
                    </Box>
                </Box>
            </form>
        </Modal>
    )
}

export default ModalCreate